@@ -1,6 +1,6 @@
 /* 
    WebDAV Class 2 locking operations
-   Copyright (C) 1999-2005, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -44,13 +44,11 @@
 #include "ne_basic.h"
 #include "ne_props.h"
 #include "ne_207.h"
-#include "ne_i18n.h"
+#include "ne_internal.h"
 #include "ne_xmlreq.h"
 
 #define HOOK_ID "http://webdav.org/neon/hooks/webdav-locking"
 
-#define EOL "\r\n"
-
 /* A list of lock objects. */
 struct lock_list {
     struct ne_lock *lock;
@@ -69,7 +67,7 @@ struct lh_req_cookie {
 
 /* Context for PROPFIND/lockdiscovery callbacks */
 struct discover_ctx {
-    ne_session *session;
+    ne_propfind_handler *phandler;
     ne_lock_result results;
     void *userdata;
     ne_buffer *cdata;
@@ -135,14 +133,14 @@ static void lk_pre_send(ne_request *r, void *userdata, ne_buffer *req)
 	struct lock_list *item;
 
 	/* Add in the If header */
-	ne_buffer_zappend(req, "If:");
+	ne_buffer_czappend(req, "If:");
 	for (item = lrc->submit; item != NULL; item = item->next) {
 	    char *uri = ne_uri_unparse(&item->lock->uri);
 	    ne_buffer_concat(req, " <", uri, "> (<",
 			     item->lock->token, ">)", NULL);
 	    ne_free(uri);
 	}
-	ne_buffer_zappend(req, EOL);
+	ne_buffer_czappend(req, "\n");
     }
 }
 
@@ -219,7 +217,7 @@ static void submit_lock(struct lh_req_cookie *lrc, struct ne_lock *lock)
 
     /* Check for dups */
     for (item = lrc->submit; item != NULL; item = item->next) {
-	if (strcasecmp(item->lock->token, lock->token) == 0)
+	if (ne_strcasecmp(item->lock->token, lock->token) == 0)
 	    return;
     }
 
@@ -243,7 +241,7 @@ struct ne_lock *ne_lockstore_findbyuri(ne_lock_store *store,
 void ne_lock_using_parent(ne_request *req, const char *path)
 {
     struct lh_req_cookie *lrc = ne_get_request_private(req, HOOK_ID);
-    ne_uri u;
+    ne_uri u = {0};
     struct lock_list *item;
     char *parent;
 
@@ -254,7 +252,6 @@ void ne_lock_using_parent(ne_request *req, const char *path)
     if (parent == NULL)
 	return;
     
-    u.authinfo = NULL;
     ne_fill_server_uri(ne_get_session(req), &u);
 
     for (item = lrc->store->locks; item != NULL; item = item->next) {
@@ -350,10 +347,7 @@ struct ne_lock *ne_lock_copy(const struct ne_lock *lock)
 {
     struct ne_lock *ret = ne_calloc(sizeof *ret);
 
-    ret->uri.path = ne_strdup(lock->uri.path);
-    ret->uri.host = ne_strdup(lock->uri.host);
-    ret->uri.scheme = ne_strdup(lock->uri.scheme);
-    ret->uri.port = lock->uri.port;
+    ne_uri_copy(&ret->uri, &lock->uri);
     ret->token = ne_strdup(lock->token);
     ret->depth = lock->depth;
     ret->type = lock->type;
@@ -408,9 +402,7 @@ int ne_unlock(ne_session *sess, const struct ne_lock *lock)
 
     ret = ne_request_dispatch(req);
     
-    if (ret == NE_OK && ne_get_status(req)->klass == 2) {
-	ret = NE_OK;    
-    } else {
+    if (ret == NE_OK && ne_get_status(req)->klass != 2) {
 	ret = NE_ERROR;
     }
 
@@ -421,7 +413,7 @@ int ne_unlock(ne_session *sess, const struct ne_lock *lock)
 
 static int parse_depth(const char *depth)
 {
-    if (strcasecmp(depth, "infinity") == 0) {
+    if (ne_strcasecmp(depth, "infinity") == 0) {
 	return NE_DEPTH_INFINITE;
     } else if (isdigit(depth[0])) {
 	return atoi(depth);
@@ -432,7 +424,7 @@ static int parse_depth(const char *depth)
 
 static long parse_timeout(const char *timeout)
 {
-    if (strcasecmp(timeout, "infinite") == 0) {
+    if (ne_strcasecmp(timeout, "infinite") == 0) {
 	return NE_TIMEOUT_INFINITE;
     } else if (strncasecmp(timeout, "Second-", 7) == 0) {
 	long to = strtol(timeout+7, NULL, 10);
@@ -444,7 +436,7 @@ static long parse_timeout(const char *timeout)
     }
 }
 
-static void discover_results(void *userdata, const char *href,
+static void discover_results(void *userdata, const ne_uri *uri,
 			     const ne_prop_result_set *set)
 {
     struct discover_ctx *ctx = userdata;
@@ -454,18 +446,16 @@ static void discover_results(void *userdata, const char *href,
     /* Require at least that the lock has a token. */
     if (lock->token) {
 	if (status && status->klass != 2) {
-	    ctx->results(ctx->userdata, NULL, lock->uri.path, status);
+	    ctx->results(ctx->userdata, NULL, uri, status);
 	} else {
-	    ctx->results(ctx->userdata, lock, lock->uri.path, NULL);
+	    ctx->results(ctx->userdata, lock, uri, NULL);
 	}
     }
     else if (status) {
-	ctx->results(ctx->userdata, NULL, href, status);
+	ctx->results(ctx->userdata, NULL, uri, status);
     }
-	
-    ne_lock_destroy(lock);
 
-    NE_DEBUG(NE_DBG_LOCKS, "End of response for %s\n", href);
+    NE_DEBUG(NE_DBG_LOCKS, "End of response for %s\n", uri->path);
 }
 
 static int 
@@ -509,8 +499,8 @@ end_element_common(struct ne_lock *l, int state, const char *cdata)
 static int end_element_ldisc(void *userdata, int state, 
                              const char *nspace, const char *name)
 {
-    struct ne_lock *lock = ne_propfind_current_private(userdata);
     struct discover_ctx *ctx = userdata;
+    struct ne_lock *lock = ne_propfind_current_private(ctx->phandler);
 
     return end_element_common(lock, state, ctx->cdata->data);
 }
@@ -638,22 +628,24 @@ static int lk_endelm(void *userdata, int state,
     return 0;
 }
 
-static void *ld_create(void *userdata, const char *href)
+/* Creator callback for private structure. */
+static void *ld_create(void *userdata, const ne_uri *uri)
 {
-    struct discover_ctx *ctx = userdata;
     struct ne_lock *lk = ne_lock_create();
 
-    if (ne_uri_parse(href, &lk->uri) != 0) {
-	ne_lock_destroy(lk);
-	return NULL;
-    }
-    
-    if (!lk->uri.host)
-	ne_fill_server_uri(ctx->session, &lk->uri);
+    ne_uri_copy(&lk->uri, uri);
 
     return lk;
 }
 
+/* Destructor callback for private structure. */
+static void ld_destroy(void *userdata, void *private)
+{
+    struct ne_lock *lk = private;
+
+    ne_lock_destroy(lk);
+}
+
 /* Discover all locks on URI */
 int ne_lock_discover(ne_session *sess, const char *uri, 
 		     ne_lock_result callback, void *userdata)
@@ -664,15 +656,13 @@ int ne_lock_discover(ne_session *sess, const char *uri,
     
     ctx.results = callback;
     ctx.userdata = userdata;
-    ctx.session = sess;
     ctx.cdata = ne_buffer_create();
+    ctx.phandler = handler = ne_propfind_create(sess, uri, NE_DEPTH_ZERO);
 
-    handler = ne_propfind_create(sess, uri, NE_DEPTH_ZERO);
-
-    ne_propfind_set_private(handler, ld_create, &ctx);
+    ne_propfind_set_private(handler, ld_create, ld_destroy, &ctx);
     
     ne_xml_push_handler(ne_propfind_get_parser(handler), 
-                        ld_startelm, ld_cdata, end_element_ldisc, handler);
+                        ld_startelm, ld_cdata, end_element_ldisc, &ctx);
     
     ret = ne_propfind_named(handler, lock_props, discover_results, &ctx);
     
@@ -705,20 +695,23 @@ int ne_lock(ne_session *sess, struct ne_lock *lock)
     ctx.cdata = ne_buffer_create();    
     ctx.req = req;
 
+    /* LOCK is not idempotent. */
+    ne_set_request_flag(req, NE_REQFLAG_IDEMPOTENT, 0);
+
     ne_xml_push_handler(parser, lk_startelm, lk_cdata, lk_endelm, &ctx);
     
     /* Create the body */
-    ne_buffer_concat(body, "<?xml version=\"1.0\" encoding=\"utf-8\"?>" EOL
-		    "<lockinfo xmlns='DAV:'>" EOL " <lockscope>",
+    ne_buffer_concat(body, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+		    "<lockinfo xmlns='DAV:'>\n" " <lockscope>",
 		    lock->scope==ne_lockscope_exclusive?
 		    "<exclusive/>":"<shared/>",
-		    "</lockscope>" EOL
+		    "</lockscope>\n"
 		    "<locktype><write/></locktype>", NULL);
 
     if (lock->owner) {
-	ne_buffer_concat(body, "<owner>", lock->owner, "</owner>" EOL, NULL);
+	ne_buffer_concat(body, "<owner>", lock->owner, "</owner>\n", NULL);
     }
-    ne_buffer_zappend(body, "</lockinfo>" EOL);
+    ne_buffer_czappend(body, "</lockinfo>\n");
 
     ne_set_request_body_buffer(req, body->data, ne_buffer_size(body));
     ne_add_request_header(req, "Content-Type", NE_XML_MEDIA_TYPE);
@@ -774,7 +767,7 @@ int ne_lock(ne_session *sess, struct ne_lock *lock)
 	    ne_set_error(sess, _("Response missing activelock for %s"), 
 			 ctx.token);
 	}
-    } else {
+    } else if (ret == NE_OK /* && status != 2xx */) {
 	ret = NE_ERROR;
     }
 
