@@ -1,6 +1,6 @@
 /* 
    Wrapper interface to XML parser
-   Copyright (C) 1999-2004, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2005, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -31,7 +31,7 @@
 #include <strings.h>
 #endif
 
-#include "ne_i18n.h"
+#include "ne_internal.h"
 
 #include "ne_alloc.h"
 #include "ne_xml.h"
@@ -47,9 +47,9 @@
 #endif
 typedef XML_Char ne_xml_char;
 
-#if !defined(XML_MAJOR_VERSION) || (XML_MAJOR_VERSION < 2 \
-                                    && XML_MINOR_VERSION == 95 \
-                                    && XML_MICRO_VERSION < 2)
+#if !defined(XML_MAJOR_VERSION)
+#define NEED_BOM_HANDLING
+#elif XML_MAJOR_VERSION < 2 && XML_MINOR_VERSION == 95 && XML_MICRO_VERSION < 2
 #define NEED_BOM_HANDLING
 #endif
 
@@ -58,9 +58,11 @@ typedef XML_Char ne_xml_char;
 #include <libxml/xmlversion.h>
 #include <libxml/parser.h>
 typedef xmlChar ne_xml_char;
-/* libxml-2.6.18 should have the fix:
- * http://bugzilla.gnome.org/show_bug.cgi?id=162613 */
+
+#if LIBXML_VERSION < 20619
+/* 2.6.19 and earlier have broken BOM handling */
 #define NEED_BOM_HANDLING
+#endif
 #else /* not HAVE_LIBXML */
 #  error need an XML parser
 #endif /* not HAVE_EXPAT */
@@ -328,7 +330,7 @@ static void start_element(void *userdata, const ne_xml_char *name,
                                   elm->nspace, elm->name, PASS_ATTS(atts));
     }
 
-    NE_DEBUG(NE_DBG_XMLPARSE, "XML: start-element (%d, {%s, %s}) => %d\n", 
+    NE_DEBUG(NE_DBG_XML, "XML: start-element (%d, {%s, %s}) => %d\n", 
              elm->parent->state, elm->nspace, elm->name, state);             
     
     if (state > 0)
@@ -353,7 +355,7 @@ static void destroy_element(struct element *elm)
 	ne_free(this_ns->uri);
 	ne_free(this_ns);
 	this_ns = next_ns;
-    };
+    }
     if (elm->default_ns)
         ne_free(elm->default_ns);
     ne_free(elm);
@@ -369,7 +371,8 @@ static void char_data(void *userdata, const ne_xml_char *data, int len)
     
     if (elm->handler->cdata_cb) {
         p->failure = elm->handler->cdata_cb(elm->handler->userdata, elm->state, data, len);
-        NE_DEBUG(NE_DBG_XML, "Cdata callback returned %d.\n", p->failure);
+        NE_DEBUG(NE_DBG_XML, "XML: char-data (%d) returns %d\n", 
+                 elm->state, p->failure);
     }        
 }
 
@@ -392,7 +395,7 @@ static void end_element(void *userdata, const ne_xml_char *name)
         }
     }
     
-    NE_DEBUG(NE_DBG_XMLPARSE, "XML: end-element (%d, {%s, %s})\n",
+    NE_DEBUG(NE_DBG_XML, "XML: end-element (%d, {%s, %s})\n",
              elm->state, elm->nspace, elm->name);
 
     /* move back up the tree */
@@ -488,23 +491,22 @@ int ne_xml_parse(ne_xml_parser *p, const char *block, size_t len)
     int ret, flag;
     /* duck out if it's broken */
     if (p->failure) {
-	NE_DEBUG(NE_DBG_XML, "Not parsing %" NE_FMT_SIZE_T " bytes.\n", 
-		 len);
+	NE_DEBUG(NE_DBG_XMLPARSE, "XML: Failed; ignoring %" NE_FMT_SIZE_T 
+                 " bytes.\n", len);
 	return p->failure;
     }
     if (len == 0) {
 	flag = -1;
 	block = "";
-	NE_DEBUG(NE_DBG_XML, "Got 0-length buffer, end of document.\n");
+	NE_DEBUG(NE_DBG_XMLPARSE, "XML: End of document.\n");
     } else {	
-	NE_DEBUG(NE_DBG_XML, "Parsing %" NE_FMT_SIZE_T " length buffer.\n",
-		 len);
+	NE_DEBUG(NE_DBG_XMLPARSE, "XML: Parsing %" NE_FMT_SIZE_T " bytes.\n", len);
 	flag = 0;
     }
 
 #ifdef NEED_BOM_HANDLING
     if (p->bom_pos < 3) {
-        NE_DEBUG(NE_DBG_XML, "Checking for UTF-8 BOM.\n");
+        NE_DEBUG(NE_DBG_XMLPARSE, "Checking for UTF-8 BOM.\n");
         while (len > 0 && p->bom_pos < 3 && 
                block[0] == BOM_UTF8[p->bom_pos]) {
             block++;
@@ -526,24 +528,25 @@ int ne_xml_parse(ne_xml_parser *p, const char *block, size_t len)
      * will already have been written in that case. */
 #ifdef HAVE_EXPAT
     ret = XML_Parse(p->parser, block, len, flag);
-    NE_DEBUG(NE_DBG_XMLPARSE, "XML_Parse returned %d\n", ret);
+    NE_DEBUG(NE_DBG_XMLPARSE, "XML: XML_Parse returned %d\n", ret);
     if (ret == 0 && p->failure == 0) {
 	ne_snprintf(p->error, ERR_SIZE,
 		    "XML parse error at line %d: %s", 
 		    XML_GetCurrentLineNumber(p->parser),
 		    XML_ErrorString(XML_GetErrorCode(p->parser)));
 	p->failure = 1;
+        NE_DEBUG(NE_DBG_XMLPARSE, "XML: Parse error: %s\n", p->error);
     }
 #else
     ret = xmlParseChunk(p->parser, block, len, flag);
-    NE_DEBUG(NE_DBG_XMLPARSE, "xmlParseChunk returned %d\n", ret);
+    NE_DEBUG(NE_DBG_XMLPARSE, "XML: xmlParseChunk returned %d\n", ret);
     /* Parse errors are normally caught by the sax_error() callback,
      * which clears p->valid. */
     if (p->parser->errNo && p->failure == 0) {
 	ne_snprintf(p->error, ERR_SIZE, "XML parse error at line %d.", 
 		    ne_xml_currentline(p));
 	p->failure = 1;
-        NE_DEBUG(NE_DBG_XMLPARSE, "XML parse error: %s\n", p->error);
+        NE_DEBUG(NE_DBG_XMLPARSE, "XML: Parse error: %s\n", p->error);
     }
 #endif
     return p->failure;
