@@ -1,6 +1,6 @@
 /* 
    sitecopy WebDAV protocol driver module
-   Copyright (C) 2000-2005, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2000-2006, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -58,6 +58,7 @@
 
 struct fetch_context {
     struct proto_file **files;
+    struct proto_file *tail;
     const char *root;
 };
 
@@ -92,14 +93,6 @@ static const ne_propname props[] = {
     { NULL }
 };
 
-static inline int get_depth(const char *href) {
-    const char *pnt;
-    int count = 0;
-    for (pnt=href; *pnt != '\0'; pnt++) /* oneliner */
-	if (*pnt == '/') count++;
-    return count;
-}
-
 /* Set session error string to 'msg: strerror(errnum)'. */
 static void syserr(ne_session *sess, const char *msg, int errnum)
 {
@@ -244,8 +237,13 @@ static int init(void **session, struct site *site)
 
     ne_set_status(sess, notify_cb, NULL);
 
-    if (site->http_limit)
-	ne_set_persist(sess, 0);
+    if (site->http_limit) {
+#if NE_VERSION_MINOR > 25
+        ne_set_session_flag(sess, NE_SESSFLAG_PERSIST, 0);
+#else
+        ne_set_persist(sess, 0);
+#endif
+    }
 
     /* Note, this won't differentiate between xsitecopy and
      * sitecopy... maybe we should put a comment in as well. */
@@ -258,6 +256,35 @@ static int init(void **session, struct site *site)
 
     ne_set_server_auth(sess, server_auth_cb, &site->server);
     
+    if (site->http_secure && site->client_cert) {
+        ne_ssl_client_cert *cc;
+
+        cc = ne_ssl_clicert_read(site->client_cert);
+        if (!cc) {
+            ne_set_error(sess, _("Could not read SSL client certificate '%s'."),
+                         site->client_cert);
+            return SITE_FAILED;
+        }
+
+        if (ne_ssl_clicert_encrypted(cc)) {
+            char password[FE_LBUFSIZ];
+
+            if (fe_decrypt_clicert(cc, password)) {
+                return SITE_FAILED;
+            }
+
+            if (ne_ssl_clicert_decrypt(cc, password) != 0) {
+                ne_set_error(sess, _("Could not decrypt SSL client "
+                                     "certificate '%s'."), site->client_cert);
+                return SITE_FAILED;
+            }
+        }
+
+        ne_ssl_set_clicert(sess, cc);
+
+        ne_ssl_clicert_free(cc);
+    }
+
     if (site->http_tolerant) {
 	/* Skip the OPTIONS, since we ignore failure anyway. */
 	return SITE_OK;
@@ -537,25 +564,24 @@ static int dir_remove(void *session, const char *dirname)
 
 /* Insert the file in the list in the appropriate position (keeping it
  * sorted). */
-static void insert_file(struct proto_file **list, struct proto_file *file)
+static void insert_file(struct fetch_context *ctx, struct proto_file *file)
 {
-    struct proto_file *previous, *current;
-    previous = NULL;
-    current = *list;
-    while (current != NULL && current->depth < file->depth) {
-	previous = current;
-	current = current->next;
-    }
-    if (previous == NULL) {
-	*list = file;
+    if (ctx->tail) {
+        ctx->tail->next = file;
     } else {
-	previous->next = file;
+        (*ctx->files) = file;
     }
-    file->next = current;
+    ctx->tail = file;
+    file->next = NULL;
 }
 
+#if NE_VERSION_MINOR > 25
+static void pfind_results(void *userdata, const ne_uri *uri,
+			  const ne_prop_result_set *set)
+#else
 static void pfind_results(void *userdata, const char *href,
 			  const ne_prop_result_set *set)
+#endif
 {
     struct fetch_context *ctx = userdata;
     struct private *private = ne_propset_private(set);
@@ -564,14 +590,15 @@ static void pfind_results(void *userdata, const char *href,
     int iscoll;
     char *uhref;
 
-    /* Find out whether this is a collection or not, then
-     * free the private structure so we don't have to worry 
-     * about it any more. */
     iscoll = private->iscollection;
-    free(private);
 
-    NE_DEBUG(NE_DBG_HTTP, "URI: [%s]: ", href);
-    
+#if NE_VERSION_MINOR < 26
+    /* For >= 0.26, this is handled by the destroy_private
+     * callback. */
+    ne_free(private);
+#endif
+
+#if NE_VERSION_MINOR < 26
     /* Strip down to the abspath segment */
     if (strncmp(href, "http://", 7) == 0)
 	href = strchr(href+7, '/');
@@ -585,11 +612,17 @@ static void pfind_results(void *userdata, const char *href,
     }
 
     uhref = ne_path_unescape(href);
+#else
+    uhref = ne_path_unescape(uri->path);
+    
+#endif
+
+    NE_DEBUG(NE_DBG_HTTP, "URI: [%s]: ", uhref);
 
     if (!ne_path_childof(ctx->root, uhref)) {
 	/* URI not a child of the root collection...  ignore this
 	 * resource */
-	NE_DEBUG(NE_DBG_HTTP, "outside root collection!\n");
+	NE_DEBUG(NE_DBG_HTTP, "not child of root collection!\n");
 	return;
     }
    
@@ -617,7 +650,6 @@ static void pfind_results(void *userdata, const char *href,
 
     file = ne_calloc(sizeof(struct proto_file));
     file->filename = ne_strdup(uhref+strlen(ctx->root));
-    file->depth = get_depth(file->filename);
     
     if (iscoll) {
 	file->type = proto_dir;
@@ -639,8 +671,7 @@ static void pfind_results(void *userdata, const char *href,
     }
 
     /* Insert the file into the files list. */
-    insert_file(ctx->files, file);
-
+    insert_file(ctx, file);
 }
 
 static int start_element(void *userdata, int parent,
@@ -663,17 +694,30 @@ static int start_element(void *userdata, int parent,
 }
 
 /* Creates the private structure. */
+#if NE_VERSION_MINOR > 25
+static void *create_private(void *userdata, const ne_uri *uri)
+#else
 static void *create_private(void *userdata, const char *uri)
+#endif
 {
     return ne_calloc(sizeof(struct private));
 }
 
+#if NE_VERSION_MINOR > 25
+static void destroy_private(void *userdata, void *private)
+{
+    struct private *priv = private;
+
+    ne_free(priv);
+}
+#endif
+
 /* TODO: optimize: only ask for lastmod + executable when we really
  * need them: it does waste bandwidth and time to ask for executable
  * when we don't want it, since it forces a 404 propstat for each
  * non-collection resource if it is not defined.  */
 static int fetch_list(void *session, const char *dirname, int need_modtimes,
-		       struct proto_file **files) 
+                      struct proto_file **files) 
 {
     ne_session *sess = session;
     int ret;
@@ -683,10 +727,15 @@ static int fetch_list(void *session, const char *dirname, int need_modtimes,
 
     ctx.root = dirname;
     ctx.files = files;
+    ctx.tail = NULL;
     ph = ne_propfind_create(sess, edirname, NE_DEPTH_ONE);
 
     /* The complex props. */
-    ne_propfind_set_private(ph, create_private, NULL);
+    ne_propfind_set_private(ph, create_private,
+#if NE_VERSION_MINOR > 25
+                            destroy_private, 
+#endif
+                            NULL);
 
     /* Register the handler for the complex props. */
     ne_xml_push_handler(ne_propfind_get_parser(ph), start_element, NULL, NULL, ph);
