@@ -1,6 +1,6 @@
 /* 
    sitecopy, manage remote web sites.
-   Copyright (C) 1998-2005, Joe Orton <joe@manyfish.co.uk>.
+   Copyright (C) 1998-2006, Joe Orton <joe@manyfish.co.uk>.
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -96,20 +96,10 @@ int fe_login(fe_login_context ctx, const char *realm, const char *hostname,
  * failures mask of 'failures' (NE_SSL_*), should be accepted. */
 int fe_accept_cert(const ne_ssl_certificate *cert, int failures);
 
-/* Modifications to the files list within
- *  site_update, site_fetch, and site_synch
- * are made after a call to fe_disable_abort(), and before
- * a call to fe_enable_abort.
- * Example:
- *  ...doing something...
- *  fe_disable_abort();
- *  ...update files list...
- *  fe_enable_abort();
- * If the process is killed while abortion is disabled, then
- * the files list is NOT guaranteed to be in a consistent state.
- */
-void fe_disable_abort(struct site *site);
-void fe_enable_abort(struct site *site);
+/* Enter password needed to decrypt given client certificate.  Returns
+ * zero on success, non-zero on failure.  'password' is of size
+ * FE_LBUFSIZ. */
+int fe_decrypt_clicert(const ne_ssl_client_cert *cert, char *password);
 
 int fe_can_update(const struct site_file *file);
 void fe_updating(const struct site_file *file);
