@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1998-2002, 2005, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
 #include <sys/types.h>
 
-#include <ne_socket.h> /* for sock_block_reader */
+#include <ne_request.h>
 
 #include "sites.h"
 #include "common.h"
@@ -44,7 +44,6 @@ struct proto_file {
     time_t modtime;
     mode_t mode;
     unsigned char checksum[16];
-    int depth;
     struct proto_file *next;
 };
 
@@ -86,7 +85,7 @@ struct proto_driver {
     int (*file_download)(void *sess, const char *local, const char *remote,
 			 int ascii);
     int (*file_read)(void *sess, const char *remote, 
-		     sock_block_reader reader, void *userdata);
+		     ne_block_reader reader, void *userdata);
 
     int (*file_delete)(void *sess, const char *filename);
     int (*file_chmod)(void *sess, const char *filename, mode_t mode);
